/*	License
 * 		inside EU: EUPL
 * 		outside EU: GPL or EUPL
 *  The program comes AS IS and without any warranty unless required by law
 *  (C) 2012 thecodemonkey
 */

package core.io;

import gui.resource.ResourceManager;

import java.io.BufferedOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;
import java.nio.charset.Charset;
import java.nio.file.DirectoryStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;

import javax.swing.JOptionPane;

import settings.Settings1;
import settings.Settings;
import util.StringUtil;

public abstract class Application implements SettingsIO {
	
	protected Path execPath,configPath, settingsPath;
	protected Settings settings;
	protected ResourceManager rm;
	public static final Charset UTF8=Charset.forName("UTF-8");
	protected enum Section{general,default_settings}
	protected Map<String, String> otherManifestVals;
	
	public static final StandardOpenOption[] DEFAULT_WRITE_OPTIONS={StandardOpenOption.CREATE,StandardOpenOption.TRUNCATE_EXISTING,StandardOpenOption.WRITE};
	
	public Application(String settingsFile) {
		this(null, settingsFile, true);
	}
	
	
	public Application(String subDir,String settingsFile,boolean localizations) {
		final String MF="manifest.dat";
		String manifestFile= subDir!=null?subDir+MF:MF;
		
		settings=new Settings1();
		URL u=ClassLoader.getSystemClassLoader().getResource(manifestFile);
		try {
			if (u==null) throw new FileNotFoundException("config files not found");
			Path p= Paths.get(u.toURI());
			execPath=p.getParent();
			String mf=p.toString();
			String ep=Matcher.quoteReplacement(mf.substring(0, mf.length()-manifestFile.length()));
			List<String> manifest=Files.readAllLines(p,UTF8);
			Section section = null;
			for (String option:manifest) {
				
				if (option.equals("[default]")) {
					section=Section.default_settings;
				} else if (option.equals("[general]")) {
					section=Section.general;
				} else {
					int k=option.indexOf('=');
					if (k>-1) {
						String s=option.substring(0,k).trim();
						String t=option.substring(k+1).trim();
	
						switch (section) {
						case general:
							if (s.equals("settingsPath")) {
								settingsPath=Paths.get(StringUtil.replaceFirst(StringUtil.replaceFirst(t,"%EXEC_PATH%",ep),"%HOME%",System.getProperty("user.home"))).resolve(settingsFile); 
							} else {
								if (otherManifestVals==null) otherManifestVals=new HashMap<>();
								otherManifestVals.put(s,t);
							}
							break;
						case default_settings:
							settings.set(s, t);
							break;
						}
					}
				}
			}
			setAdditionalDefaultSettings();
			loadSettings();
			configPath=execPath.resolve("cfg");
			if (localizations){
				rm=new gui.resource.ResourceManager2(this,settings.getLocale("locale"));
			}
		} catch (Exception e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, e.getMessage());
			throw new RuntimeException(e);
		}
	}
	
	protected void setAdditionalDefaultSettings() {
		
	}
	
	public Settings getSettings() {
		return settings;
	}
		
	public Path getExecPath() {
		return execPath;		
	}
	
	public ResourceManager getResourceManager() {
		return rm;
	}
	
	public List<Locale> getLocales() {
		LinkedList<Locale> l=new LinkedList<Locale>();
		Path localizationDir=execPath.resolve("locale");
		DirectoryStream<Path> stream=null;
		try {
			stream=Files.newDirectoryStream(
			localizationDir,new DirectoryStream.Filter<Path>() {

				@Override
				public boolean accept(Path entry) throws IOException {
					String fn=entry.getFileName().toString();
					
					return fn.endsWith(".properties") && fn.charAt(0)!='.';
				}
			});
			Iterator<Path> it=stream.iterator();
			while (it.hasNext()) {
				Path f=it.next();
				
				String s=f.getFileName().toString();
				if (s.startsWith("_")) {
					l.add(new Locale(s.substring(1, 3)));
				} else {
					l.add(new Locale(s.substring(0, 2),s.substring(3, 5)));
				}
			}
			return l;
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		} finally {
			try {
				stream.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	
	public void saveSettings() throws IOException {
		BufferedOutputStream out=new BufferedOutputStream(Files.newOutputStream(settingsPath,StandardOpenOption.CREATE,StandardOpenOption.TRUNCATE_EXISTING,StandardOpenOption.WRITE));
		Set<String> ks=settings.getSettingsKeyset();
		for (String name:ks) {
			String s=settings.toString(name);
			if (s==null) {
				System.out.println(name);
			} else {
				write(out,name);
				write(out,"=");
				write(out,s);
				write(out,"\n");
			}
		}
		out.close();
	}
	
	private void write(BufferedOutputStream out,String s) throws IOException {
		out.write(s.getBytes(UTF8));
	}
	
	public void loadSettings() throws IOException {
		
		if (Files.exists(settingsPath)) {
			List<String> settingList= Files.readAllLines(settingsPath,UTF8);
			
			for (String s:settingList) {
				int k=s.indexOf('=');
				if (k>-1) {
					String name=s.substring(0,k).trim();
					String value=s.substring(k+1).trim();
					settings.set(name, value);
				}
			}
			
		} 
	}
	
	public void quit() {
		try {
			saveSettings();
		} catch (IOException e) {
			e.printStackTrace();
		}
		System.exit(0);
	}
	
	public synchronized void updateManifest(String name,String newValue) throws IOException {
		if (newValue!=null) {
			Path mf=execPath.resolve("manifest.dat");
			List<String> list=Files.readAllLines(mf, UTF8);
			BufferedOutputStream out= new BufferedOutputStream(Files.newOutputStream(mf, DEFAULT_WRITE_OPTIONS));
			for (String s:list) {
				int k=s.indexOf('=');
				if (k>-1) {
					String n0=s.substring(0,k).trim();
					String value=s.substring(k+1).trim();
					out.write(n0.getBytes(UTF8));
					out.write('=');
					if (n0.equals(name)) {
						out.write(newValue.getBytes(UTF8));
					} else {
						out.write(value.getBytes(UTF8));
					}
					
				} else {
					out.write(s.getBytes(UTF8));
				}out.write('\n');
			}
			out.close();
			otherManifestVals.put(name, newValue);
		} else {
			throw new IllegalArgumentException();
		}
	}
	
	public String getManifestValue(String name) {
		if (otherManifestVals!=null)
			return otherManifestVals.get(name);
		else
			//throw new NoSuchElementException();
			return null;
	}
	
	public Path getConfigurationPath() {
		return configPath;
	}
	
	public abstract String getVersion(); 
}
