/*	License
 * 		inside EU: EUPL
 * 		outside EU: GPL or EUPL
 *  The program comes AS IS and without any warranty unless required by law
 *  (C) 2012 thecodemonkey
 */

package gui.settings.dialog.component;

import java.util.Collection;
import java.util.Locale;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JOptionPane;

import settings.Settings;

public class LanguageSettingsComponent extends ComboBoxSettingsComponent<LanguageSettingsComponent.GuiLocale> {
	private DefaultComboBoxModel<GuiLocale> cbModel;
	private GuiLocale lastLocaleSetting;
	
	public LanguageSettingsComponent(String settingsName,Collection<Locale> locales) {
		super(settingsName);
		cbModel=(DefaultComboBoxModel<GuiLocale>) super.cb.getModel();
		for (Locale l:locales){
			cbModel.addElement(new GuiLocale(l));
		}
	}
	
	@Override
	public void saveSettings(Settings s) {
		GuiLocale currentSetting=cb.getModel().getElementAt(cb.getSelectedIndex());
		s.set(settingsName, currentSetting.getLocale());
		
		if (!currentSetting.equals(lastLocaleSetting)) {
			JOptionPane.showMessageDialog(null, "Localisation changed, please restart application");
		}
	}

	@Override
	public void loadSettings(Settings s) {
		lastLocaleSetting=new GuiLocale(s.getLocale(settingsName));
		cbModel.setSelectedItem(lastLocaleSetting);
	}
	
	protected class GuiLocale {
		private Locale l;
		
		public GuiLocale(Locale l) {
			this.l=l;
		}
		
		public Locale getLocale() {
			return l;
		}
		
		public String toString() {
			return l.getDisplayLanguage();
		};
		
		@Override
		public boolean equals(Object obj) {
			if (obj instanceof GuiLocale) {
				return l.equals(((GuiLocale)obj).l);
			} else {
				return false;
			}
		}
		
		@Override
		public int hashCode() {
			return l.hashCode();
		}
	}
//
//	public void setAvailableLocales(Collection<Locale> locales) {
//		
//	}
}
