/*	License
 * 		inside EU: EUPL
 * 		outside EU: GPL or EUPL
 *  The program comes AS IS and without any warranty unless required by law
 *  (C) 2012 thecodemonkey
 */

package settings;


import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.LinkedHashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Observable;
import java.util.Set;

public class Settings1 extends Observable implements Settings{

	private Map<String, String> settings=new LinkedHashMap<>();
	
	@Override
	public void fireSettingsChanged(String name) {
		setChanged();
		notifyObservers(name);
	}
	
	@Override
	public void set(String name,String value) {
		settings.put(name, value);
	}
	
	@Override
	public void set(String name,int value) {
		settings.put(name, Integer.toString(value));
	}
	
	@Override
	public void set(String name,Locale l) {
		settings.put(name, l.getLanguage());
	}
	
	@Override
	public void set(String name,String[] values) {
		if (values.length>0) {
			StringBuilder sb=new StringBuilder();

			for (int i=0;i<values.length-1;i++) {
				sb.append(values[i]);
				sb.append(';');
			}
			sb.append(values[values.length-1]);
			settings.put(name,sb.toString());
		}
		
	}
	
	@Override
	public void set(String name,int[] values) {
		if (values.length>0) {
			StringBuilder sb=new StringBuilder();

			for (int i=0;i<values.length-1;i++) {
				sb.append(Integer.toString(values[i]));
				sb.append(';');
			}
			sb.append(values[values.length-1]);
			settings.put(name, sb.toString());
		}
	}
	
	@Override
	public void set(String name,boolean value) {
		settings.put(name,value?"1":"0");
	}
	
	@Override
	public void set(String name,Path value) {
		if (value!=null)
			settings.put(name,value.toString());
	}
	
	@Override
	public String getString(String name) {
		return  settings.get(name);
	}
	
	@Override
	public int[] getIntArray(String name) {
		String v= settings.get(name);
		try {
			if (v!=null) {
				String[] s=v.split(";");
				int[] intArray=new int[s.length];
				int i=0;
				
				for (String s0:s) {
					intArray[i++]=Integer.parseInt(s0);
				}
				return intArray;
			} else {
				return null;
			}
		} catch (NumberFormatException nfe){
			return null;
		}
	}
	
	@Override
	public String[] getStringArray(String name) {
		String v= settings.get(name);
		
		if (v!=null) {
			return v.split(";");
		} else {
			return null;
		}
	}
	
	@Override
	public Integer getInt(String name) {
		try {
			return Integer.valueOf( settings.get(name));
		} catch (NumberFormatException nfe) {
			return null;
		}
	}
	
	@Override
	public boolean getBoolean(String name) {
		String b=settings.get(name);
		if (b==null) 
			return false;
		else
			return b.equals("1");
	}
	
	@Override
	public Double getDouble(String name) {
		try {
			return Double.valueOf( settings.get(name));
		} catch (NumberFormatException nfe) {
			return null;
		}
	}
	
	@Override
	public Locale getLocale(String name) {
		return new Locale( settings.get(name));
	}
	
	
	
	@Override
	public Path getPath(String name) {
		String p= settings.get(name);
		if (p!=null&&p.length()>0)
			return Paths.get(p);
		else
			return null;
	}

	@Override
	public void set(String name, double value) {
		settings.put(name, Double.toString(value));
	}

	@Override
	public Set<String> getSettingsKeyset() {
		return settings.keySet();
	}

	@Override
	public String toString(String name) {
		return settings.get(name);
	}
}
